# SYNOPSIS
#
#   AX_PROFILING
#
# DESCRIPTION
#   This macro defines an 'enable' configure option for passing compiler options  
#   to compile with gprof symbols.  If --enable-gprof is passed to 
#   configure then the variable PROFILING is set to true and PROFILING_FLAGS is set 
#   to '-pg', else PROFILING is false and PROFILING_FLAGS is set to ''.
# 
#   To compile with the PROFILING_FLAGS variable add it to the AM_*FLAGS variable 
#   in Makefile.am as follows:
# 
#      AM_CXXFLAGS += $(PROFILING_FLAGS)
#

AC_DEFUN([AX_PROFILING],[
AC_PREREQ(2.59)

AC_ARG_ENABLE([gprof],
  [AC_HELP_STRING([--enable-gprof],[Turn on gprof profiling])],
  [DISTCHECK_CONFIGURE_FLAGS="$DISTCHECK_CONFIGURE_FLAGS --enable-gprof=\"${enable_gprof}\""]
)

AS_IF([test "x$enable_gprof" = "xyes"], 
      [PROFILING_FLAGS="-pg -DPROFILING"
       AC_SUBST([PROFILING_FLAGS])])

])dnl AX_PROFILING
