# SYNOPSIS
#
#   AX_DEBUG
#
# DESCRIPTION
#
#   This macro allows the user to compile the code in debug mode. The argument
#   to the --with-debug=<DebugLevel>. The DebugLevel parameter can be -g0 -g1 or -g2
#   If the passed parameter is not one of those, -g0 will be turned on by default
#
#   To compile with the DEBUG_FLAGS variable add it to the AM_*FLAGS variable
#   in Makefile.am as follows:
#
#      AM_CXXFLAGS += $(DEBUG_FLAGS)
#

AC_DEFUN([AX_DEBUG],[
AC_PREREQ(2.59)

AC_ARG_WITH([debug],
  [AC_HELP_STRING([--with-debug=[0|1|3]], [Specify the debug level as 0, no debug information, 1, minimal debug information, 2, normal debug info, or 3, maximum debug info. @<:@default=0@:>@])],
  [DEBUG_LEVEL=$withval
   DISTCHECK_CONFIGURE_FLAGS="$DISTCHECK_CONFIGURE_FLAGS --with-debug=\"${withval}\""
  ])

case "x$DEBUG_LEVEL" in
    x) AC_SUBST([DEBUG_MISSING],[true]);;
    x0|x1|x2|x3)
      DEBUG_FLAGS="-g${DEBUG_LEVEL} -DDEBUG"
      AC_SUBST([DEBUG_FLAGS])
      AC_SUBST([DEBUG_MISSING],[false])
      ;;
    x*)   
      AC_MSG_ERROR([--with-debug requires the debug level as 0, no debug information, 1, minimal debug information, 2, normal debug info, or 3, maximum debug info.])
      AC_SUBST([DEBUG_MISSING],[true])
      ;; 
esac    

])dnl AX_DEBUG
