/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#include<iostream>
#include "IPComMan.h"

using std::cout;
using std::endl;

//example of static message size point-to-point communications using the IPComMan API
int main(int argc, char **argv) 
{
  int iRank, iSize;

  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &iRank);
  MPI_Comm_size(MPI_COMM_WORLD, &iSize);

  IPComMan *CM = new IPComMan(MPI_COMM_WORLD, 0, 10240, IPComMan::Neighbors);

  std::set<int> SetNbrs;
  for (int iNbr = 0; iNbr < iSize; ++iNbr)
  {
    if (iNbr == iRank)
      continue;
    SetNbrs.insert(iNbr);
  }
  CM->set_Nbrs(SetNbrs);

  int iMsgSize = sizeof(int);
  CM->set_fixed_msg_size(iMsgSize);

  int* piMsgSend = (int*)CM->alloc_msg(sizeof(int));

  for (int iPid = 0; iPid < iSize; ++iPid)
  {
    *piMsgSend = iRank*10;
    CM->send((iRank+1)%iSize, (void*)piMsgSend);
  }

  CM->finalize_send();
  CM->free_msg(piMsgSend);

  void *pvMsgRecv;
  int iPidFrom;
  while(int rc = CM->receive(pvMsgRecv, &iPidFrom))
  {
    int iMsg = *((int*)pvMsgRecv);
    cout << "pid " << iRank << "got the message " << iMsg << " from pid " << iPidFrom << "\n";
    CM->free_msg(pvMsgRecv);
  }

  delete CM;
  MPI_Finalize();
}
