/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#include <iostream>
#include <cstdlib>
#include <mpi.h>

int main(int argc, char**argv)
{
  if (argc != 4) { std::cout << "usage: " << argv[0] << " <iterations> <tag> <num_neighbors>\n"; }
  int iterations =  atoi(argv[1]);
  int TAG =         atoi(argv[2]);
  int iNumNbrs =    atoi(argv[3]);
  int rank;
  int size;
  int i;
  int iTotMsg = iterations*iNumNbrs;
  double *message_send = new double[iTotMsg];
  double *message_recv = new double[iTotMsg];
  MPI_Init(NULL,NULL);
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
  MPI_Comm_size(MPI_COMM_WORLD,&size);
  MPI_Request* reqs_send = new MPI_Request[iTotMsg];
  MPI_Request* reqs_recv = new MPI_Request[iTotMsg];

  int *to = new int[iNumNbrs];
  int *from = new int[iNumNbrs];

  for (int iNbr = 1; iNbr <= iNumNbrs; iNbr++)
  {
    to[iNbr-1] = (rank + iNbr) % size;
    from[iNbr-1] = (rank + size - iNbr) % size;
  }

  double timer = MPI_Wtime();

  for (i=0; i < iterations; ++i)
  {
    for (int iNbr = 0; iNbr < iNumNbrs; iNbr++)
    {
      MPI_Isend(&message_send[iNumNbrs*i + iNbr], 1, MPI_DOUBLE, to[iNbr], TAG, MPI_COMM_WORLD, &reqs_send[iNbr + iNumNbrs*i]);
    }
  }

  for (i=0; i < iterations; ++i)
  {
    for (int iNbr = 0; iNbr < iNumNbrs; iNbr++)
    {
      MPI_Irecv(&message_recv[iNumNbrs*i + iNbr], 1, MPI_DOUBLE, from[iNbr], TAG, MPI_COMM_WORLD, &reqs_recv[iNbr + iNumNbrs*i]);
    }
  }


  double timer_before_waitall = MPI_Wtime() - timer;

  MPI_Waitall(iTotMsg, reqs_send, MPI_STATUSES_IGNORE);
  MPI_Waitall(iTotMsg, reqs_recv, MPI_STATUSES_IGNORE);

  double timer_before_reqs_delete = MPI_Wtime() - timer;

  delete [] reqs_send;
  delete [] reqs_recv;
  delete [] to;
  delete [] from;

  double timer_before_barrier = MPI_Wtime() - timer;

  MPI_Barrier(MPI_COMM_WORLD);
  timer = MPI_Wtime() - timer;

  if (rank == 0)
  {
    std::cout << "Time before waitall spent: " << timer_before_waitall << "sec\n";
    std::cout << "Time before reqs delete: " << timer_before_reqs_delete << "sec\n";
    std::cout << "Time before barrier: " << timer_before_barrier << "sec\n";
    std::cout << "Time spent: " << timer << "sec\n";
  }

  MPI_Finalize();
  return 0;
}
