/******************************************************************************

  (c) 2009-2011 Scientific Computation Research Center,
      Rensselaer Polytechnic Institute. All rights reserved.

  The LICENSE-SCOREC file included with this distribution describes the terms
  of the SCOREC Non-Commercial License this program is distributed under.

*******************************************************************************/

#ifdef USE_IPCOMMAN
#include <tiled_ipcomman.h>
typedef TiledIPComMan implementation;
#else
#include <tiled_mpi.h>
typedef TiledMPI implementation;
#endif

#include <iostream>

int main(int argc, char** argv)
{
#ifdef USE_IPCOMMAN
  if (argc < 3) { std::cout << "usage: " << argv[0] << " <iterations> <buffer size>\n"; }
#else
  if (argc < 2) { std::cout << "usage: " << argv[0] << " <iterations> <tile_side>\n"; }
#endif
  int iterations = atoi(argv[1]);
  int iBufSize;
  if (argc == 3)
    iBufSize = atoi(argv[2]);

  TilePattern pattern;
  HardCodedTilePattern1(pattern);

  implementation impl;

  MPI_Init(NULL,NULL);
#ifdef USE_IPCOMMAN
  impl.Setup(pattern, iBufSize, sizeof(double));
#else
  impl.Setup(pattern, iterations);
#endif

  double send_message = 3.14159;
  double recv_message;

  double timer = MPI_Wtime();

#ifdef USE_IPCOMMAN
  for (int i=0; i < iterations; ++i)
  {
    impl.SendToNeighbors(&send_message, sizeof(double));
  }
  impl.Finalize();
  impl.ReceiveFromNeighbors();
#else
  for (int i=0; i < iterations; ++i)
  {
    impl.SendToNeighbors(&send_message, sizeof(double), i);
    impl.ReceiveFromNeighbors(&recv_message, sizeof(double), i);
  }
  impl.Finalize();
#endif

  MPI_Barrier(MPI_COMM_WORLD);
  timer = MPI_Wtime() - timer;

  int iRank;
  MPI_Comm_rank(MPI_COMM_WORLD, &iRank);
  if (iRank == 0)
    std::cout << "Time spent: " << timer << "sec\n";

  MPI_Finalize();
  return 0;
}

